/**

 \file ecGene.h
 \brief contains convenient, optional gene classes
 \todo XXX Unfortunately, we need to work out passive vs. active gene issues

 $Id: ecGene.h,v 1.11 2001/05/01 18:02:28 mcoletti Exp $

*/
#ifndef EC_GENE_H
#define EC_GENE_H


#include <iostream>


/// Abstract base class for genes
/**
   \deprecated

   This is not in use yet.
 */
class
ecGene
{
   public:

      virtual ~ecGene() {}

      /// returns probability that the given gene should be mutated
      virtual float probability() = 0;


   protected: 

      /// used by ostream<< to print gene values
      virtual void print_( std::ostream & ) const = 0;

      friend std::ostream& operator<<( std::ostream & os, ecGene const & gene );

}; // ecGene




/// convenient ecGene hierarchy print function
/**
   \deprecated

   This is not in use yet.
 */
std::ostream&
operator<<( std::ostream & os, ecGene const & gene )
{
   gene.print_( os );

   return os;
} // operator<<( std::ostream & os, ecGene const & gene )



/// Abstract base class for real genes
/**
   \deprecated

   This is not in use yet.
 */
class
ecRealGene : public ecGene
{
   public:

      //@{
      /// default upper and lower bounds
      static const float lower_bound_default;
      static const float upper_bound_default;
      //@}

      virtual ~ecRealGene() {}

      ecRealGene() 
         : lower_( lower_bound_default ),
           upper_( upper_bound_default )
      {}

      //@{
      /// Establishes the gene's lower bounds
      float  lower() const { return lower_; }
      float& lower()       { return lower_; }
      //@}

      //@{
      /// Establishes the gene's upper bounds
      float  upper() const { return upper_; }
      float& upper()       { return upper_; }
      //@}

      //@{
      /// Establishes the gene's actual value
      float  value() const { return value_; }
      float& value()       { return value_; }
      //@}

      /// operator to assign a value to an ecRealGene
      /**
         operator to assign a value to an ecRealGene that insures the
         new value is clamped to [lower,upper] 
      */
      ecRealGene const & operator=( float );

      ///
      virtual void print_( std::ostream & os ) const
      {
         os << value();         
      } // print_

   private:

      /// lower bound
      float lower_;

      /// upper bound
      float upper_;

      /// actual gene value
      float value_;

}; // ecRealGene


const float ecRealGene::lower_bound_default = -10.0;
const float ecRealGene::upper_bound_default =  10.0;




/// Abstract base class for real genes used for gaussian style mutation
/**
   \deprecated

   This is not in use yet.
 */
class
ecGaussianGene : public ecRealGene
{
   public:

      /// default starting scale factor for real gaussian mutation
      static const float default_scale;

      ecGaussianGene() 
         : scale_( ecGaussianGene::default_scale )
      {}

      virtual ~ecGaussianGene() 
      {}

      float  scale() const { return scale_;}
      float& scale()       { return scale_;}


   private:

      /// base pertubation value factor 
      float scale_;

}; // ecGaussianGene




/// default starting scale factor for real gaussian mutation
/**
   \deprecated

   This is not in use yet.
 */
const float ecGaussianGene::default_scale = 0.1;



/// Abstract base class for binary genes
/**
   \todo XXX may not be necessary given std::vector<bool> and std::bitset
   \deprecated

   This is not in use yet.
 */
class
ecBinaryGene : public ecGene
{
   public:

      virtual ~ecBinaryGene() {}

}; // ecBinaryGene


#endif
