//
// ecHalt.h
//
// $Id: ecHalt.h,v 1.5 2000/08/02 14:01:22 jbassett Exp $
//

#ifndef EC_HALT_H
#define EC_HALT_H


template<class Population>
class ecHalt
{
   public:

      ecHalt() {}

      virtual ~ecHalt() {}


      virtual bool operator()( Population & p ) const = 0;

      virtual bool reset() = 0;

}; // class ecHalt



/**! A convenience halting function oriented to stopping after so many
 generations.
*/
template<class Population>
class ecGenerationHalt
    : public ecHalt<Population>
{
   public:

      ecGenerationHalt() 
         : generations_( 0 ),
           target_generations_( 0 )
      {}


      explicit ecGenerationHalt( int tg ) 
         : generations_( 0 ),
           target_generations_( tg )
      {}

      //!
      virtual bool operator()( Population & p ) const
      {
         if ( generations_ == target_generations_ )
            return true;

         generations_++;

         return false;
      }

      //!
      virtual bool reset() 
      {
         generations_ = 0;

         // XXX Why does this return a bool?  Is this right?
         return true;
      }

      //!
      int generations() const  { return generations_; }
      //!
      int generations( int g ) { return generations_ = g; }

      //!
      int targetGenerations() const  { return target_generations_; }
      //!
      int targetGenerations( int g ) { return target_generations_ = g; }

   private:

      /**! how many generations we've executed so far; mutable because
       halt functions are const parameters for ecEA, yet we want to
       update this
      */
      mutable int generations_;

      //! total number of generations we want to run for
      int target_generations_;
      

}; // class ecGenerationHalt




/**! A convenience halting function oriented to stopping after so many
 births.
*/
template<class Population>
class ecBirthHalt
    : public ecHalt<Population>
{
   public:

      ecBirthHalt() 
         : target_births_( 0 )
      {}


      explicit ecBirthHalt( int tg ) 
         : target_births_( tg )
      {}

      //!
      virtual bool operator()( Population & p ) const
      {
         if ( p.births() == target_births_ )
            return true;

         return false;
      }

      //!
      virtual bool reset() 
      {
         // NOP

         // XXX Why does this return a bool?  Is this right?
         return true;
      }

   private:

      //! total number of births we want to run until
      int target_births_;

}; // class ecBirthHalt




#endif
