//
// ecPopulation.h
//
// $Id: ecPopulation.h,v 1.7 2000/07/26 19:48:41 mcoletti Exp $
//

#ifndef EC_POPULATION_H
#define EC_POPULATION_H

#include <algorithm>
#include <deque>


/// A container of individuals.  Some statistical information is also kept.
/**
 Essentially an adaptor design pattern that adds births and deaths
 state to a given container of individuals.
*/
#ifndef __STL_LIMITED_DEFAULT_TEMPLATES
template <class Individual, class Sequence = deque<Individual> >
#else   
template <class Individual, class Sequence>
#endif
class ecPopulation : public Sequence
{
   public:

      ecPopulation()
      {}

      ecPopulation( size_t n )
         : Sequence( n )    // start with n new individuals
      {}

      virtual ~ecPopulation()
      {}

      typedef typename Sequence::value_type value_type;
      typedef typename Sequence::size_type size_type;
      typedef typename Sequence::reference reference;
      typedef typename Sequence::const_reference const_reference;
      typedef typename Sequence::iterator iterator;
      typedef typename Sequence::const_iterator const_iterator;

}; // ecPopulation




/// helper functor for randomize()
template<class Individual>
struct randomize__
{
      void operator()( Individual & i ) { i.randomize(); }
}; // struct randomize__



/// generic algorithm for randomizing individuals in the given range
template<class ForwardIterator>
void
randomize( ForwardIterator begin__, ForwardIterator end__ )
{
   std::for_each( begin__, end__, 
                  randomize__<typename ForwardIterator::value_type>() );
} // randomize


#endif

