//
// eaRandom.cpp
//

#include <eclib/ecRandom.h>

#include <cmath>
#include <cstdlib>
#include <climits>

static const char* ident_ = 
"$Id: ecRandom.cpp,v 1.7 2000/07/28 17:40:33 jbassett Exp $";


#ifdef HAVE_RANDOM
#define RANDOM (random())
#define SRANDOM(x) (srandom(x))
#define RANDOM_MAX INT_MAX
#else
#define RANDOM (rand())
#define SRANDOM(x) (srand(x))
#define RANDOM_MAX RAND_MAX
#endif

void
init_rand( int seed )
{
  SRANDOM(seed);
} // init_rand




int
irand(int n)
{
  return ( RANDOM % n );
} // irand
 


float
frand(void)
{
  return ( (float) RANDOM / (float) RANDOM_MAX );
} // frand



double
drand(void)
{
  return ( (double) RANDOM / (double) RANDOM_MAX );
} // drand



double
gauss()
{
  static int iset = 0;
  static double gset;
  double fac, r, v1, v2;

  if (iset == 0)		/* We don't have an extra deviate handy  */
    {
      /*  Pick two uniform numbers in the square extending from -1  */
      /*  to +1 in each direction.			 	  */

      do {
        v1 = 2.0 * drand() - 1.0;
        v2 = 2.0 * drand() - 1.0;
        r = v1*v1 + v2*v2;	/* See if they are in the unit circle */
      }  while (r >= 1.0);      /* and if they are not, try again.    */

      fac = sqrt(-2.0*log(r)/r);

      /*  Now make the Box-Muller transformation to get two normal deviates. */
      /*  Return one and save the other for next time.		       */

      gset = v1 * fac;
      iset = 1;		/* Set flag	*/
      return (v2 * fac);
    }
  else			/* We have an extra deviate handy, so return that */
    {
      iset = 0;		/*  Unset the flag.		*/
      return gset;
    }
}  // double gauss()




float log2(float x)
{
  return 3.32194 * log10(x);       /* log2(10) = 3.32194  (approx) */
}




