//
// ecReproStrategy.h
//
// $Id: ecReproStrategy.h,v 1.6 2000/07/31 22:00:43 jbassett Exp $
//

#ifndef EC_REPROSTRATEGY_H
#define EC_REPROSTRATEGY_H

#include <algorithm>
#include <iterator>

#ifndef EC_STRATEGY_H
#include <eclib/ecStrategy.h>
#endif


/// A functor that creates a child population from parents.
/**
 A simple creation "functor" used by ecEA and its derivatives.  The
 children container should already be sized to the number of children
 to be generated.

 \note

 A \e functor is just a class that defines an \c operator member that
 accepts zero or more arguments such that an object of that class can
 be used like a regular ole function call.

  */
template< class Population, class BirthOperator >
class
ecReproductionStrategy : 
   public ecStrategyWithOperator<Population,BirthOperator>
{
   public:

      ecReproductionStrategy( BirthOperator & bop ) 
         :  ecStrategyWithOperator<Population,BirthOperator>( bop )
      { }

      virtual ~ecReproductionStrategy() {}


      /// Create children from the given parents
      /**
         Create children from given parents using the birth operator
         bound to this reproduction strategy.  The std::generate()
         applies the birth operator to each child. The birth operator
         is accessible via the op() member function. */
      virtual void operator()( Population & parents,
                               Population & children )
      {
         // give birth_operator_ a chance to set up
         init( parents );      

         // create the kids via the birth operator
         //std::generate( children.begin(), children.end(), op() );

         // generate() has problems calling virtual functions.
         typename Population::iterator i;
         for (i = children.begin(); i < children.end(); ++i)
             *i = op()();

         // give birth_operator_ a chance to do some form of
         // post-processing, if any
         cleanup( parents, children ); 

      } // function ReproductionStrategy


}; // functor ecReproductionStrategy


// XXX Add similar ReproductionStrategys to support different creation
// XXX methodologies; e.g., the ones Paul, Jeff, and Guido suggested


#endif
