//
// ecSurvivalStrategy.h
//
// $Id: ecSurvivalStrategy.h,v 1.1 2000/08/02 14:17:13 jbassett Exp $
//

#ifndef EC_SURVIVALSTRATEGY_H
#define EC_SURVIVALSTRATEGY_H

#include <algorithm>
#include <iterator>


#ifndef EC_STRATEGY_H
#include <eclib/ecStrategy.h>
#endif



//!
template< class Population, class SurvivalOperator >
class ecSurvivalStrategy : 
   public ecStrategyWithOperator<Population,SurvivalOperator>
{
   public:

      ecSurvivalStrategy( SurvivalOperator & so )
         :  ecStrategyWithOperator<Population,SurvivalOperator>( so )
      {}

      virtual ~ecSurvivalStrategy() {}

      //!
      virtual void cull( Population  & parents,
                         Population  & children,
                         Population  & survivors ) = 0;

}; // class ecSurvivalStrategy



//!
template< class Population, class SurvivalOperator >
class ecMuPlusLambdaSurvivalStrategy : 
   public ecSurvivalStrategy<Population,SurvivalOperator>
{
   public:

      ecMuPlusLambdaSurvivalStrategy( SurvivalOperator & so )
         :  ecSurvivalStrategy<Population,SurvivalOperator>( so )
      {}

      virtual ~ecMuPlusLambdaSurvivalStrategy() {}


      //!
      virtual void cull( Population  & parents,
                 Population  & children,
                 Population  & survivors ) 
      {
         // initialize the survival operator with both the parents
         // and children; this should put the survival operator in
         // "(mu + lambda)" mode since we're using the init_() that takes
         // both populations as arguments
         init( parents, children );

         // create the survivors
         //generate( survivors.begin(), survivors.end(), op() );

         // generate() has problems calling virtual functions.
         typename Population::iterator i;
         for (i = survivors.begin(); i != survivors.end(); ++i)
             *i = op()();

         // clean up anything that needs, well, cleaning up
         cleanup( parents, children );
      }

}; // class ecMuPlusLambdaSurvivalStrategy



//!
template< class Population, class SurvivalOperator >
class ecMuCommaLambdaSurvivalStrategy : 
   public ecSurvivalStrategy<Population,SurvivalOperator>
{
   public:

      ecMuCommaLambdaSurvivalStrategy( SurvivalOperator & so )
         :  ecSurvivalStrategy<Population,SurvivalOperator>( so )
      {}

      virtual ~ecMuCommaLambdaSurvivalStrategy() {}

      //!
      virtual void cull( Population  & parents,
                         Population  & children,
                         Population  & survivors ) 
      {
         // Yes, the parents are ignored because that's what happens
         // in a (mu,lambda) EA.  I.e., the parents are "killed".

         // initialize the survival operator with just the children;
         // this should put the survival operator in "(mu + lambda)"
         // mode since we're using the init_() that takes just the
         // children as the argument; the parents parameter for this
         // member is left in for API consistency.
         init( children );

         // create the survivors
         //generate( survivors.begin(), survivors.end(), op() );

         // generate() has problems calling virtual functions.
         typename Population::iterator i;
         for (i = survivors.begin(); i != survivors.end(); ++i)
             *i = op()();

         // clean up anything that needs, well, cleaning up
         cleanup( children );
      }

}; // class ecMuCommaLambdaSurvivalStrategy


#endif
