//
// ecClone_t.cpp
//
// $Id: ecClone_t.cpp,v 1.3 2000/10/18 19:13:04 mcoletti Exp $
//


#include <algorithm>
#include <iterator>
#include <functional>
#include <numeric>
#include <iostream>
#include <cassert>


using namespace std;


#include <eclib/ecIndividual.h>
#include <eclib/ecPopulation.h>
#include <eclib/ecRandom.h>
#include <eclib/ecBirth.h>
#include <eclib/ecSelector.h>


// A test individual class for the x^2 landscape
//
// Used to exercise ecIndividual oriented tests only.  More
// sophisticated tests are in other test programs.
//
class SimpleIndividual : public ecIndividual< float, float >
{
   public:

      SimpleIndividual()
         : ecIndividual< float, float >( 2 )
      {}
      // Individuals of this type will have two elements in their genome

      void randomize()
      {
         // randomize the genes to values in [0,4]
         for ( iterator i = begin(); i != end(); i++ )
         {
            *i = drand() * 4;
         }
      }

      float evaluate() const
      {
         return accumulate( begin(), end(), 1.0, multiplies<float>() );
      }


   protected:

      void print_( ostream & os ) const
      {
         os << "( ";

         ostream_iterator<float> ostr_itr( os, " " );
         copy( begin(), end(), ostr_itr );

         os << "), "
            << fitness() 
            << "\n";
      }

}; // SimpleIndividual


// Add some syntactic sugar so we don't have to type long names all the time

typedef ecPopulation<SimpleIndividual>     Population;
typedef ecSequentialSelector< Population > SequentialSelector;




// Create an initial parent population.  Then create a child
// population.  We arbitrarily choose the gauassian mutation birth
// operator to create the child population.
//
void
init( size_t num_parents, 
      size_t num_children,
      Population & parents, 
      Population & children )
{

   // let's add some guys to the parents
   for ( int i = 0; i < 10; i++ )
   {
      parents.push_back( SimpleIndividual() );
   }

   ostream_iterator<SimpleIndividual> ostr_itr( cout, "\n" );

   // ... and stir them up a bit

   cout << "initial parents:\n";

   randomize( parents.begin(), parents.end() );

   copy( parents.begin(), parents.end(), ostr_itr );


   // now cut loose the gaussian mutation operator to create a child
   // population
   
   cout << "\n... and a child population generated via gaussian mutation:\n";

   
   SequentialSelector selector;
   selector.init( parents );

   generate_n( back_inserter( children ),
               20,
               ecGaussianMutationBirthOperator< Population, SequentialSelector >( selector, 0.5, 0.75 ) );


   copy( children.begin(), children.end(), ostr_itr );

} // init()




int
main( int argc, char** argv )
{
   ecPopulation<SimpleIndividual> parents, children;

   init( 10, 20, parents, children );

   

   SequentialSelector selector;

   ecCloneBirthOperator< Population, SequentialSelector >
      cloner( selector );

   cloner.init( parents );

   generate_n( back_inserter( children ),
               20, 
               cloner );
               

   cout << "generated kids:\n";

   ostream_iterator<SimpleIndividual> ostr_itr( cout, "\n" );

   copy( children.begin(), children.end(), ostr_itr );

   exit( 0 );
}

