//
// ecGenOp_t.cpp
//
// $Id: ecGenOp_t.cpp,v 1.3 2000/10/18 19:13:05 mcoletti Exp $
//


#include <algorithm>
#include <iterator>
#include <functional>
#include <numeric>
#include <iostream>
#include <cassert>
#include <vector>


using namespace std;


#include <eclib/ecIndividual.h>
#include <eclib/ecPopulation.h>
#include <eclib/ecRandom.h>
#include <eclib/ecBirth.h>
#include <eclib/ecSelector.h>
#include <eclib/ecGenOp.h>


// A test individual class for the 1-max landscape
//
// Used to exercise ecIndividual oriented tests only.  More
// sophisticated tests are in other test programs.
//
class BinaryIndividual : public ecIndividual< bool, float, vector<bool> >
{
   public:

      BinaryIndividual()
         : ecIndividual< bool, float, vector<bool> >( 30 )
      {}
      // Individuals of this type will have two elements in their genome

      void randomize()
      {
         // randomize the genes to values in [0,4]
         for ( iterator i = begin(); i != end(); i++ )
         {
            *i = irand(2);
         }
      }

      void notEvaluated()
      {}

      float evaluate() const
      {
         return accumulate( begin(), end(), 1.0 );
      }


   protected:

      void print_( ostream & os ) const
      {
         os << "( ";

         ostream_iterator<float> ostr_itr( os, " " );
         copy( begin(), end(), ostr_itr );

         os << "), "
            << fitness() 
            << "\n";
      }

}; // BinaryIndividual


// Add some syntactic sugar so we don't have to type long names all the time

typedef ecPopulation<BinaryIndividual>       Population;
typedef ecSequentialSelector< Population >   SequentialSelector;
typedef ecGeneticOperator<BinaryIndividual>  GeneticOperator;
typedef vector< GeneticOperator* >           Pipeline;




// Create an initial parent population.  Then create a child
// population.  We arbitrarily choose the gauassian mutation birth
// operator to create the child population.
//
void
init( size_t num_parents, 
      Population & parents)
{

   // let's add some guys to the parents
   for ( int i = 0; i < num_parents; i++ )
   {
      parents.push_back( BinaryIndividual() );
   }

   ostream_iterator<BinaryIndividual> ostr_itr( cout, "\n" );

   // ... and stir them up a bit

   cout << "initial parents:\n";

   //randomize( parents.begin(), parents.end() );

   copy( parents.begin(), parents.end(), ostr_itr );

} // init()




int
main( int argc, char** argv )
{
   ecPopulation<BinaryIndividual> parents, children;

   init( 10, parents );


   SequentialSelector selector;
   selector.init(parents);

   ecBitFlipMutationOperator<BinaryIndividual>  mutate(0.03);
   ecNPointCrossoverOperator<BinaryIndividual>  cross(2, 0.8);

   Pipeline  pipeline;
   pipeline.push_back(&mutate);
   pipeline.push_back(&cross);


   ecPipelineBirthOperator< Population, SequentialSelector, Pipeline >
      pipelineOp( selector, pipeline );

   pipelineOp.init( parents );

   generate_n( back_inserter( children ),
               20, 
               pipelineOp );
               

   cout << "generated kids:\n";

   ostream_iterator<BinaryIndividual> ostr_itr( cout, "\n" );

   copy( children.begin(), children.end(), ostr_itr );

   exit( 0 );
}

