//
// ecIndividual_t.cpp
//
// $Id: ecIndividual_t.cpp,v 1.4 2000/10/18 19:13:05 mcoletti Exp $
//


#include <algorithm>
#include <functional>
#include <numeric>
#include <iostream>
#include <cassert>


using namespace std;


#include <eclib/ecIndividual.h>
#include <eclib/ecPopulation.h>
#include <eclib/ecRandom.h>
#include <eclib/ecBirth.h>


// A test individual class for the x^2 landscape
//
// Used to exercise ecIndividual oriented tests only.  More
// sophisticated tests are in other test programs.
//
class SimpleIndividual : public ecIndividual< float, float >
{
   public:

      SimpleIndividual()
         : ecIndividual< float, float >( 2 )
      {}
      // Individuals of this type will have two elements in their genome

      void randomize()
      {
         // randomize the genes to values in [0,4]
         for ( iterator i = begin(); i != end(); i++ )
         {
            *i = drand() * 4;
         }
      }

      float evaluate() const
      {
         return accumulate( begin(), end(), 1.0, multiplies<float>() );
      }


   protected:

      void print_( ostream & os ) const
      {
         os << "( ";

         ostream_iterator<float> ostr_itr( os, " " );
         copy( begin(), end(), ostr_itr );

         os << "), "
            << fitness() 
            << "\n";
      }

}; // SimpleIndividual



int
main( int argc, char** argv )
{
   // test ctor
   SimpleIndividual individual;

   // test that the initial values are all zero
   cout << individual << endl;

   // now stir up the genes
   individual.randomize();

   // ... and verify that, indeed, they are stirred up; _and_ that the
   // fitness is calculated correctly
   cout << individual << endl;

   // now apply a simple gaussian mutatation to the genes
   // perturb each child component (i.e., gene)
   for_each( individual.begin(), 
             individual.end(), 
             ecGaussianMutateGene<SimpleIndividual::value_type>( 0.5 ) );

   cout << individual << endl;

   for_each( individual.begin(), 
             individual.end(), 
             ecGaussianMutateGene<SimpleIndividual::value_type>( 5 ) );

   cout << individual << endl;

   exit( 0 );

}

