//
// ecPopulation_t.cpp
//
// $Id: ecPopulation_t.cpp,v 1.4 2000/10/18 19:13:05 mcoletti Exp $
//


#include <algorithm>
#include <functional>
#include <numeric>
#include <iostream>
#include <cassert>


using namespace std;


#include <eclib/ecIndividual.h>
#include <eclib/ecPopulation.h>
#include <eclib/ecRandom.h>
#include <eclib/ecBirth.h>


// A test individual class for the x^2 landscape
//
// Used to exercise ecIndividual oriented tests only.  More
// sophisticated tests are in other test programs.
//
class SimpleIndividual : public ecIndividual< float, float >
{
   public:

      SimpleIndividual()
         : ecIndividual< float, float >( 2 )
      {}
      // Individuals of this type will have two elements in their genome

      void randomize()
      {
         // randomize the genes to values in [0,4]
         for ( iterator i = begin(); i != end(); i++ )
         {
            *i = drand() * 4;
         }
      }

      float evaluate() const
      {
         return accumulate( begin(), end(), 1.0, multiplies<float>() );
      }


   protected:

      void print_( ostream & os ) const
      {
         os << ", ( ";

         ostream_iterator<float> ostr_itr( os, " " );
         copy( begin(), end(), ostr_itr );

         os << "), "
            << fitness() 
            << "\n";
      }

}; // SimpleIndividual



int
main( int argc, char** argv )
{
   ecPopulation<SimpleIndividual> population;

   // shove in ten simple individuals into a population
   for ( int i = 0; i < 10; i++ )
   {
      population.push_back( SimpleIndividual() );
   }

   // blat them all out
   ostream_iterator<SimpleIndividual> si_ostr( cout, "\n" );
   copy( population.begin(), population.end(), si_ostr );

   // stir up the individuals and print them again
   randomize( population.begin(), population.end() );

   copy( population.begin(), population.end(), si_ostr );

   exit( 0 );

}

