//
// ecCreate_t.cpp
//
// $Id: ecReproStrategy_t.cpp,v 1.4 2000/10/18 19:13:05 mcoletti Exp $
//


#include <algorithm>
#include <iterator>
#include <functional>
#include <numeric>
#include <iostream>
#include <cassert>


using namespace std;


#include <eclib/ecIndividual.h>
#include <eclib/ecPopulation.h>
#include <eclib/ecRandom.h>
#include <eclib/ecBirth.h>
#include <eclib/ecSelector.h>
#include <eclib/ecHalt.h>
#include <eclib/ecReproStrategy.h>
#include <eclib/ecSurvive.h>


// A test individual class for the x^2 landscape
//
// Used to exercise ecIndividual oriented tests only.  More
// sophisticated tests are in other test programs.
//
class SimpleIndividual : public ecIndividual< float, float >
{
   public:

      SimpleIndividual()
         : ecIndividual< float, float >( 2 )
      {}
      // Individuals of this type will have two elements in their genome

      void randomize()
      {
         // randomize the genes to values in [0,4]
         for ( iterator i = begin(); i != end(); i++ )
         {
            *i = drand() * 4;
         }
      }

      float evaluate() const
      {
         return accumulate( begin(), end(), 1.0, multiplies<float>() );
      }


   protected:

      void print_( ostream & os ) const
      {
         os << "( ";

         ostream_iterator<float> ostr_itr( os, " " );
         copy( begin(), end(), ostr_itr );

         os << "), "
            << fitness() 
            << "\n";
      }

}; // SimpleIndividual


typedef ecPopulation<SimpleIndividual> Population;


int
main( int argc, char** argv )
{
   Population parents;

   // let's add some guys to the parents
   for ( int i = 0; i < 10; i++ )
   {
      parents.push_back( SimpleIndividual() );
   }

   ostream_iterator<SimpleIndividual> ostr_itr( cout, "\n" );

   // give them initial random values
   randomize( parents.begin(), parents.end() );

   copy( parents.begin(), parents.end(), ostr_itr );


   Population children( 20 ); // want 20 kids



   typedef ecSequentialSelector< Population > SSelector;

   typedef ecGaussianMutationBirthOperator< Population, SSelector > GaussianBOP;

   {
      SSelector sequential_selector;
      sequential_selector.init( parents );

      GaussianBOP gaussian_bop( sequential_selector, 0.5, 0.75 );


      ecReproductionStrategy< Population, GaussianBOP >  
         rs( gaussian_bop );

      // now create the children with the given create function

      rs( parents, children );

   }




   cout << "gaussian children:\n";
   copy( children.begin(), children.end(), ostr_itr );



   // now try the uniform only policy; not by itself useful, but
   // useful for checking its correctness

   typedef ecUniformSelector< Population > USelector;

   typedef ecUniformCrossoverBirthOperator< Population, USelector > 
      UniformCrossoverBOP;

   {

      USelector uniform_selector;
      uniform_selector.init( parents );

      UniformCrossoverBOP uniform_bop( uniform_selector );


      ecReproductionStrategy< Population, UniformCrossoverBOP > 
         uniform_create( uniform_bop );

      uniform_create( parents, children );

   }


   cout << "uniform crossover children:\n";
   copy( children.begin(), children.end(), ostr_itr );


   // now try the uniform only policy; not by itself useful, but
   // useful for checking its correctness


   typedef ecUniformMutationBirthOperator<Population, USelector> 
      UniformCrossoverMutationBOP;

   {

      USelector selector;
      selector.init(parents);

      UniformCrossoverMutationBOP uniform_mutation_bop( selector,
                                                     0.5, // scale
                                                     0.75 // probability
         );


      ecReproductionStrategy< Population, UniformCrossoverMutationBOP > 
      uniform_mutation_create( uniform_mutation_bop );

      uniform_mutation_create( parents, children );

   }


   cout << "uniform crossover mutation children:\n";
   copy( children.begin(), children.end(), ostr_itr );

   exit( 0 );
}



